import 'package:hl_lieferservice/model/delivery.dart';

abstract class DeliveryState {}

class DeliveryInitial extends DeliveryState {}

class DeliveryLoaded extends DeliveryState {
  DeliveryLoaded({required this.delivery});

  Delivery delivery;

  DeliveryLoaded copyWith(Delivery? delivery) {
    return DeliveryLoaded(delivery: delivery ?? this.delivery);
  }
}

class DeliveryFinished extends DeliveryState {
  DeliveryFinished({required this.delivery});

  Delivery delivery;

  DeliveryFinished copyWith(Delivery? delivery) {
    return DeliveryFinished(delivery: delivery ?? this.delivery);
  }
}
